__author__ = 'mackaiver'
import time
import requests
import requests.exceptions
import logging

logger = logging.getLogger(__name__)


class FetcherException(Exception):
    def __init__(self, messages):
        self.messages = messages


class HtmlFetcher:
    #take departure and arrival location and optinaly departure day and time
    def get_efa_html(self, dep, arr, day=None, departure_time=None):
        raise NotImplementedError


class DBHtmlFetcher(HtmlFetcher):
    _url = 'http://mobile.bahn.de/bin/mobil/query2.exe/dox'

    def get_efa_html(self, dep, arr, day=None, departure_time=None):
        if not day:
            day = time.strftime("%d.%m.%Y")

        if not departure_time:
            departure_time = time.strftime("%H:%M")

        payload = {'REQ0HafasOptimize1': '0:1',
                   'REQ0HafasSearchForw': '1',
                   'REQ0JourneyStopsS0A': '1',
                   'REQ0JourneyDate': day,
                   'REQ0JourneyStopsS0G': dep,
                   'REQ0JourneyStopsS0ID': None,
                   'REQ0JourneyStopsZ0A': '1',
                   'REQ0JourneyStopsZ0G': arr,
                   'REQ0JourneyStopsZ0ID': None,
                   'REQ0JourneyTime': departure_time,
                   'REQ0Tariff_Class': '2',
                   'REQ0Tariff_TravellerReductionClass.1': '0',
                   'REQ0Tariff_TravellerType.1': 'E',
                   'existOptimizePrice': '1',
                   'immediateAvail': 'ON',
                   'queryPageDisplayed': 'yes',
                   'start': 'Suchen'}
        try:
            r = requests.post(self._url, data=payload)
            if not r.text:
                logger.debug('Response text is empty.')
                raise FetcherException('Response text is empty.')

            return r.text

        except requests.exceptions.RequestException as e:
            logger.debug('RequestException ' + str(e))
            raise FetcherException(str(e))


#! /usr/bin/python3
# -*- coding: utf-8 -*-
__author__ = 'mackaiver'

#Lets start with a simple commandline tool
import argparse
import os

from colorama import init, Fore, Style

from seins.PageParser import DBPageParser, PageContentError
from seins.HtmlFetcher import FetcherException

#init colorama so it works on windows as well.
#The autoreset flag keeps me from using RESET on each line I want to color
init(autoreset=True)

import logging
#create a logger for this module
logger = logging.getLogger(__name__)
#the usual formatter
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
# create a handler and make it use the formatter
handler = logging.StreamHandler()
handler.setFormatter(formatter)
# now tell the logger to use the handler
logger.addHandler(handler)
logger.propagate = False


def is_valid_file(parser, arg):
    (folder, t) = os.path.split(arg)
    #logger.debug('given path is:' + os.path.split(arg))

    if not folder == '' and not os.path.exists(folder):
        parser.error("The folder %s does not exist!" % folder)
    else:
        return arg


def parse_args():
    p = argparse.ArgumentParser(description='Lecker data fetching from EFA via the commandline. ',
                                formatter_class=argparse.ArgumentDefaultsHelpFormatter)

    p.add_argument('-d', default='Universität s-Bahnhof, Dortmund', metavar='--departing_station', type=str,
                   help='Name of the departing station')
    p.add_argument('-a', default='Dortmund hbf', metavar='--arrival_station', type=str,
                   help='Name of the arrival station')

    p.add_argument('-o', metavar='--output', type=lambda path: is_valid_file(p, path), help='path to outputfile')
    p.add_argument('-v', action="store_true", help='Show some nice debug and info logging output')
    p.add_argument('-s', action="store_true", help='only display S-Bahn connections')

    args = p.parse_args()
    #check for debug logging
    if args.v:
        logging.basicConfig(level=logging.DEBUG)
    else:
        logging.basicConfig(level=logging.WARN)

    return args.o, args.d, args.a, args.s


def main():
    (output_path, departure, arrival, sbahn_only) = parse_args()
    connections = []

    try:
        page = DBPageParser(departure, arrival)
        connections = page.connections
        if output_path:
            with open(output_path, 'wt') as file:
                file.write(page.html)
                logger.info("Output written to " + output_path)

    except PageContentError as e:
        logger.error('Webpage returned an error message: ' + str(e))
    except FetcherException as e:
        logger.error('Fetcher could not get valid response from server: ' + str(e))

    #do some pretty printing
    print('------------ Connections from: ' + Style.BRIGHT + departure + Style.RESET_ALL +
          '  to: ' + Style.BRIGHT + arrival)

    print(' departure, arrival, delay, connection')
    for (d, a, delay, t) in connections:
        if sbahn_only and not t.strip() is 'S':
            continue

        if delay and int(delay) >= 5:
            print(d + ',    ' + a + ',    ' + Fore.RED + delay + Fore.RESET + ',    ' + t)
        else:
            print(d + ',    ' + a + ',    ' + str(delay) + ',    ' + t)


if __name__ == '__main__':
    main()




__author__ = 'mackaiver'

import logging

from bs4 import BeautifulSoup

from seins.HtmlFetcher import DBHtmlFetcher


logger = logging.getLogger(__name__)


class PageContentError(Exception):
    def __init__(self, messages):
        self.messages = messages


class PageParser:
    _html = None
    _soup = None

    def __init__(self, dep, arr, day=None, departure_time=None):
        fetcher = DBHtmlFetcher()
        self._html = fetcher.get_efa_html(dep, arr, day, departure_time)
        self._soup = BeautifulSoup(self._html)

    @property
    def connections(self):
        raise NotImplementedError

    @property
    def errors(self):
        raise NotImplementedError


class DBPageParser(PageParser):
    _errormessages = []
    _connections = []

    def __init__(self, dep, arr, day=None, departure_time=None):
        super().__init__(dep, arr, day, departure_time)

    @classmethod
    def from_html(cls, html):
        cls._html = html
        cls._soup = BeautifulSoup(cls._html)

    @classmethod
    def from_html_fetcher(cls, fetcher, dep, arr, day=None, departure_time=None):
        cls._html = fetcher.get_efa_html(dep, arr, day, departure_time)
        cls._soup = BeautifulSoup(cls._html)

    #returns a tuple of the form (departuretime, arrivaltime, delay, traintype)
    @property
    def connections(self):
        if self.errors:
            raise PageContentError(self.errors)

        if not self._connections:
            self._connections = self._parse_trains_()

        return self._connections

    #returns a list of strings hopefully containing meaningfull erromessages from the webpage we parsed
    @property
    def errors(self):
        if self._errormessages:
            return self._errormessages

        errortags = self._soup.find_all("div", {"class": "fline errormsg"})
        logger.debug('Error tags:  ' + str(errortags))
        self._errormessages = [e.text for e in errortags]
        return self._errormessages

    @property
    def html(self):
        return self._html

    def _parse_trains_(self):
        trains = []
        arrivals = self._soup.select("tr.ovConLine")
        logger.debug('table rows with trains:  ' + str(arrivals))

        for t in arrivals:
            trains.append(self._parse_row(t))

        return trains

    def _parse_row(self, row):
        #print(str(row))
        dep = None
        arr = None
        delay = None
        traintype = None

        cell1 = row.select("td.timelink  a")
        if cell1:
            dep = cell1[0].text[:5]
            arr = cell1[0].text[5:]
        cell2 = row.select("td.tprt  span")
        if cell2:
            delay = cell2[0].text
        cell3 = row.select("td.iphonepfeil")
        if cell3:
            traintype = cell3[0].text

        return dep, arr, delay, traintype



from setuptools import setup
from setuptools import find_packages

setup(
    name = 'seins',
    packages = find_packages() ,
    version = '0.3',
    description = 'A  small module and  command line utility to show you when the '
                  'next train to your desired location is arriving by parsing the DB website',
    author = 'Kai',
    author_email = 'kai@woistbier.de',
    install_requires = [ 'colorama', 'requests', ' beautifulsoup4' ],
    url = 'https://github.com/mackaiver/sEins-Server',   # use the URL to the github repo
    download_url = 'https://github.com/mackaiver/leckerBahn-Server/archive/master.zip',
    keywords = ['traffic', 'scrapping', 'utility', 'transportation', 's1', 'DB'],
    license = 'MIT',
    classifiers = [
        "Development Status :: 3 - Alpha",
        "Topic :: Utilities",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
    ],
    entry_points = {
        "console_scripts": [
           "s1 = cmd_line.seins_cmd:main",
           "seins = cmd_line.seins_cmd:main",
    ],
    }
)


__author__ = 'mackaiver'

from colorama import init, Fore, Style
#init colorama so it works on windows as well.
#The autoreset flag keeps me from using RESET on each line I want to color
init(autoreset=True)

import logging
#create a logger for this module
logger = logging.getLogger(__name__)
#use colorama codes to color diferent output levels
logging.addLevelName(logging.INFO, Style.BRIGHT +
                                      logging.getLevelName(logging.INFO) + Style.RESET_ALL)
logging.addLevelName(logging.WARNING, Style.BRIGHT + Fore.YELLOW +
                                      logging.getLevelName(logging.WARNING) + Style.RESET_ALL)
logging.addLevelName(logging.ERROR, Style.BRIGHT + Fore.RED + logging.getLevelName(logging.ERROR) + Style.RESET_ALL)
#the usual formatter
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
# create a handler and make it use the formatter
handler = logging.StreamHandler()
handler.setFormatter(formatter)
# now tell the logger to use the handler
logger.addHandler(handler)
logger.propagate = False